import {fakeAsync, getTestBed, TestBed} from '@angular/core/testing';

import { NoteService } from './note.service';
import {HttpClientTestingModule, HttpTestingController} from '@angular/common/http/testing';
import {PatientContextService} from '../services/patient-context.service';
import {Note} from '@nspop/gm-web-facade-api';
import {of} from 'rxjs/internal/observable/of';
import {Observable} from 'rxjs';

describe('NoteService', () => {
  let injector: TestBed;
  let service: NoteService;
  let httpMock: HttpTestingController;
  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
      providers: [
      ],
    });
    service = TestBed.inject(NoteService);
    injector = getTestBed();
    httpMock = TestBed.inject(HttpTestingController);
  });
  afterEach(() => {
    httpMock.verify();
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  it('should call http post when we attempt to create a new note', fakeAsync(() => {
    let postResult: boolean;
    service.createNoteForActivity('1234', {content: 'some content', authorName: 'John Doe'} as Note)
      .subscribe(value => postResult = value);

    const req = httpMock.expectOne(`http://localhost:8280/graviditetsmappen/api/maternity/v1/note/1234`);
    expect(req.request.method).toBe('POST');
    req.flush(of(true));
  }));

  it('should return immediately on create when the resource body is empty', fakeAsync(() => {
    service.createNoteForActivity( '1234', {} as Note).subscribe(value => {
      expect(value).toBe('Empty resource, nothing to post');
    });
  }));

  it('should return immediately on update when the resource body is empty', fakeAsync(() => {
    service.update({}, '1234').subscribe(value => {
      expect(value).toBe('Empty resource, nothing to post');
    });
  }));
  describe('createOrUpdate tests', () => {
    it('should return an empty promise if function is called with a note that has no content or if the is no activity id passed', () => {
      service.createOrUpdateNote(undefined, {careplanId: '1234', content: 'Hello World'} as any, new Date())
        .subscribe(value => expect(value).toEqual(true));
      service.createOrUpdateNote('1234', {careplanId: '1234', content: ''} as any, new Date())
        .subscribe(value => expect(value).toEqual(true));
    });

    it('should update if the passed in note has an id', () => {
      spyOn(service, 'update').and.callThrough();
      service.createOrUpdateNote('1337',
        {careplanId: '1234', content: 'Hello World', documentUniqueId: 'fooId'} as any, new Date());
      expect(service.update).toHaveBeenCalled();
    });

    it('should create a new note if passed note does not have an id', () => {
      spyOn(service, 'createNoteForActivity').and.callThrough();
      service.createOrUpdateNote('1337',
        {careplanId: '1234', content: 'Hello World'} as any, new Date());
      expect(service.createNoteForActivity).toHaveBeenCalled();
    });
  });
});
