import { ComponentFixture, TestBed } from '@angular/core/testing';

import { CheckboxSingleComponent } from './checkbox-single.component';
import {AngularMaterialModule} from '../modules/angular-material.module';
import {MatCheckboxChange} from '@angular/material/checkbox';

describe('CheckboxSingleComponent', () => {
  let component: CheckboxSingleComponent;
  let fixture: ComponentFixture<CheckboxSingleComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [ CheckboxSingleComponent ],
      imports: [AngularMaterialModule]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(CheckboxSingleComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    component.registerOnChange(() => 'change registered');
    component.registerOnTouched(() => 'touch registered');
    component.makeDirtyable();
    expect(component).toBeTruthy();
  });

  it('should invoke change callback if dirtyable', () => {
    let valueChanged;
    component.registerOnChange((val: boolean) => valueChanged = val);
    component.makeDirtyable();
    component.writeValue(true);
    expect(valueChanged).toBeTrue();
  });

  it('should set its value when new values are entered', () => {
    component.writeValue(true);
    expect(component.value).toBeTrue();
  });

  it('should handle changes', () => {
    component.handleChange({checked: true} as MatCheckboxChange);
    expect(component.value).toBeTrue();
  });

  it('should set value to undefined when checked is false', () => {
    component.handleChange({checked: false} as MatCheckboxChange);
    expect(component.value).toBeUndefined();
  });
});
