import { Component, Input, OnInit } from '@angular/core';
import { Observable } from 'rxjs';
import { ClinicalObservationsService } from './clinical-observations.service';
import { ObservationTable } from '@nspop/gm-web-facade-api';
import { DataCardService } from '../../api/data-card.service';
import { tap } from 'rxjs/operators';
import { ObservationFormType } from '../measurements-modal/measurements-modal.component';

@Component({
  selector: 'app-clinical-exams',
  templateUrl: './clinical-exams.component.html',
  styleUrls: [ './clinical-exams.component.scss', './tables.scss' ]
})
export class ClinicalExamsComponent implements OnInit {
  @Input() public showRelatedActivityModal: { isOpen: boolean; activityId?: string; observationType?: ObservationFormType } | undefined;
  @Input() public measurementsModal: { isOpen: boolean; observationType: ObservationFormType } | undefined;

  public tableObservable$: Observable<ObservationTable | undefined> = this.clinicalObservationsService.observationGroups.pipe(
    tap(results => {
      results?.mother?.rows?.sort((a, b) => (b?.date && a?.date ? b.date.getTime() - a.date.getTime() : 0));
      results?.fetuses?.forEach((fetus => {
        fetus.rows?.sort((a, b) => (b?.date && a?.date ? b.date.getTime() - a.date.getTime() : 0));
      }));
    })
  );

  constructor(
    public clinicalObservationsService: ClinicalObservationsService,
    public dataCardService: DataCardService
  ) { }

  ngOnInit(): void {
  }

  public createMeasurements(): void {
    if (this.measurementsModal) {
      this.measurementsModal.observationType = 'regular';
      this.measurementsModal.isOpen = true;
    }
  }

  public showActivity(activityId: string): void {
    if (this.showRelatedActivityModal) {
      this.showRelatedActivityModal.activityId = activityId;
      this.showRelatedActivityModal.isOpen = true;
      this.showRelatedActivityModal.observationType = 'regular';
    }
  }
}
