import { Component, OnInit, ViewChild } from '@angular/core';
import { DataCard } from '@nspop/gm-web-facade-api';
import { NgForm } from '@angular/forms';
import { FormComponent } from '../../confirm-navigation-if-dirty.framework';
import { DataCardService } from '../../api/data-card.service';
import { copy } from '../../../utils/copy';
import { NoteModalComponent } from '../../activities/note-modal/note-modal.component';
import { ActivitiesService } from '../../activities/activities.service';
import { ObservationFormType } from '../measurements-modal/measurements-modal.component';

@Component({
  selector: 'app-measurements-tab',
  templateUrl: './measurements-tab.component.html',
  styleUrls: [ './measurements-tab.component.scss' ]
})
export class MeasurementsTabComponent extends FormComponent implements OnInit {
  @ViewChild('noteModal') public noteModal: NoteModalComponent | undefined;
  public dataCard: DataCard | undefined;
  public isSubmitting = false;
  public ultraSoundSaveListeners: (() => any)[] = [];
  public showRelatedActivityModal: { isOpen: boolean; activityId?: string; observationType?: ObservationFormType } = { isOpen: false };
  public measurementsModal: { isOpen: boolean; observationType: ObservationFormType } = { isOpen: false, observationType: 'regular' };
  @ViewChild('form', { static: true }) form: NgForm | undefined;
  private documentUniqueId = '';
  private version = '';

  constructor(private dataCardService: DataCardService, private activitiesService: ActivitiesService) { super(); }

  ngOnInit(): void {
    this.dataCardService.selectedDataCard$.subscribe((dataCard?: DataCard) => {
      if (dataCard) {
        this.documentUniqueId = dataCard.documentUniqueId ?? '';
        this.version = dataCard.version ?? '';
        this.dataCard = copy(dataCard);
      }
    });
    this.activitiesService.openNoteModalEvent$.subscribe((modalOptions) => {
      this.noteModal?.setNoteModalState(true, modalOptions.activity, modalOptions.allowActivityChange);
    });
  }

  onUltrasoundSave(): void {
    this.ultraSoundSaveListeners.forEach(f => f());
  }

  public save(): void {
    if (this.form?.valid && this.dataCard) {
      this.isSubmitting = true;
      this.dataCardService.update(this.dataCard, this.documentUniqueId, this.version)
        .subscribe(
          () => {
            this.dataCardService.refresh()
              .then(() => {
                this.isSubmitting = false;
                this.submitDone(window.scrollY);
              });
          },
          () => {
            this.isSubmitting = false;
          }
        );
    } else {
      console.error('Could not submit form, since it is invalid.');
    }
  }
}
