import {ComponentFixture, fakeAsync, flush, TestBed} from '@angular/core/testing';
import {PregnancyComponent} from './pregnancy.component';
import {of} from 'rxjs';
import {AngularMaterialModule} from '../modules/angular-material.module';
import {ControlValueAccessor, FormsModule, NG_VALUE_ACCESSOR, ReactiveFormsModule} from '@angular/forms';
import {MatIconTestingModule} from '@angular/material/icon/testing';
import {DataCardService} from '../api/data-card.service';
import {createDataCardServiceSpy} from '../api/data-card.service.spec';
import {getPropertyAsSpy} from '../../utils/spy';
import SpyObj = jasmine.SpyObj;
import {Component, forwardRef} from '@angular/core';
import {HeaderButtonsComponent} from '../tabs/tabs.component';
import {DataCard} from '@nspop/gm-web-facade-api';

const APP_MOCK_OVERALL_ASSESSMENT_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => MockOverallAssesmentComponent),
  multi: true
};
@Component({
  selector: 'app-overall-assessment',
  template: '',
  providers: [APP_MOCK_OVERALL_ASSESSMENT_ACCESSOR]
})
class MockOverallAssesmentComponent implements ControlValueAccessor{
  registerOnChange(fn: any): void {
  }
  registerOnTouched(fn: any): void {
  }
  writeValue(obj: any): void {
  }
}

const APP_MOCK_MEDICAL_HISTORY_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => MockMedicalHistoryComponent),
  multi: true
};
@Component({
  selector: 'app-medical-history',
  template: '',
  providers: [APP_MOCK_MEDICAL_HISTORY_ACCESSOR]
})
class MockMedicalHistoryComponent implements ControlValueAccessor{
  registerOnChange(fn: any): void {
  }
  registerOnTouched(fn: any): void {
  }
  writeValue(obj: any): void {
  }
}

const APP_MOCK_PREV_PREGNANCIES_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => MockPrevPregnanciesComponent),
  multi: true
};
@Component({
  selector: 'app-previous-pregnancies',
  template: '',
  providers: [APP_MOCK_PREV_PREGNANCIES_ACCESSOR]
})
class MockPrevPregnanciesComponent implements ControlValueAccessor{
  registerOnChange(fn: any): void {
  }
  registerOnTouched(fn: any): void {
  }
  writeValue(obj: any): void {
  }
}

const APP_MOCK_SCREENINGS_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => MockScreeningsComponent),
  multi: true
};
@Component({
  selector: 'app-screenings',
  template: '',
  providers: [APP_MOCK_SCREENINGS_ACCESSOR]
})
class MockScreeningsComponent implements ControlValueAccessor{
  registerOnChange(fn: any): void {
  }
  registerOnTouched(fn: any): void {
  }
  writeValue(obj: any): void {
  }
}

describe('PregnancyComponent', () => {
  let component: PregnancyComponent;
  let fixture: ComponentFixture<PregnancyComponent>;
  let dataCardServiceStub: SpyObj<DataCardService>;

  beforeEach(async () => {

    dataCardServiceStub = createDataCardServiceSpy();
    dataCardServiceStub.update.and.returnValue(of({} as any));
    dataCardServiceStub.refresh.and.returnValue(of({} as any).toPromise());
    getPropertyAsSpy(dataCardServiceStub, 'isDataCardClosed').and.returnValue(of(false));
    getPropertyAsSpy(dataCardServiceStub, 'selectedDataCard$').and.returnValue(of({documentStatus: 'ACTIVE'} as DataCard));

    await TestBed.configureTestingModule({
      declarations: [
        PregnancyComponent,
        MockMedicalHistoryComponent,
        MockPrevPregnanciesComponent,
        MockOverallAssesmentComponent,
        MockScreeningsComponent,
        HeaderButtonsComponent
      ],
      imports: [
        AngularMaterialModule,
        ReactiveFormsModule,
        FormsModule,
        MatIconTestingModule,
      ],
      providers: [
        {provide: DataCardService, useValue: dataCardServiceStub}
      ]
    })
      .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(PregnancyComponent);
    component = fixture.componentInstance;
  });

  it('should create', () => {
    fixture.detectChanges();
    expect(component).toBeTruthy();
  });

  it('disables save button while saving', fakeAsync(() => {
    fixture.detectChanges();
    expect(component.isSubmitting).toBeFalse();
    component.save();
    expect(component.isSubmitting).toBeTrue();
    flush();
    expect(component.isSubmitting).toBeFalse();
  }));

  it('handles that active datacard could be undefined', fakeAsync(() => {
    getPropertyAsSpy(dataCardServiceStub, 'selectedDataCard$').and.returnValue(of(undefined));
    fixture.detectChanges();
    flush();
    expect(component.dataCard).toBeUndefined();
  }));
});
