import {Component, forwardRef} from '@angular/core';
import {ControlContainer, ControlValueAccessor, NG_VALUE_ACCESSOR, NgForm} from '@angular/forms';
import {DataCard} from '@nspop/gm-web-facade-api';
import deepEqual from 'deep-equal';
import {DataCardWrapper} from '../../model-wrappers/dataCard.wrapper';
import { calculateOffsetDate } from '../../../utils/dates';

export const APP_SCREENINGS_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => ScreeningsComponent),
  multi: true
};

@Component({
  selector: 'app-screenings',
  templateUrl: './screenings.component.html',
  styleUrls: ['./screenings.component.scss'],
  providers: [APP_SCREENINGS_CONTROL_VALUE_ACCESSOR],
  viewProviders: [ { provide: ControlContainer, useExisting: NgForm } ]
})
export class ScreeningsComponent implements ControlValueAccessor {
  private innerModel: DataCardWrapper| undefined;
  private dirtyAble = false;
  private onChangeCallback: (newValue: DataCard|undefined) => any = () => {};
  private onTouchedCallback: () => any = () => {};

  constructor() { }

  get value(): DataCardWrapper|undefined {
    return this.innerModel;
  }

  set value(value: DataCardWrapper|undefined) {
    if (!deepEqual(value?.asDataCard, this.innerModel?.asDataCard)) {
      this.innerModel = value;
      if (this.dirtyAble) {
        this.onChangeCallback(value?.asDataCard);
      }
      if (!this.dirtyAble && this.innerModel) { // We don't want to mark the field as dirty while data gets initialized
        this.dirtyAble = true;
      }
    }
  }

  registerOnChange(fn: any): void {
    this.onChangeCallback = fn;
  }

  registerOnTouched(fn: any): void {
    this.onTouchedCallback = fn;
  }

  writeValue(newValue: DataCard): void {
    this.value = new DataCardWrapper(newValue);
  }

  public hepatitisDateChanged(date: Date): void {
    if (this.areAllDatesUndefined()) {
      this.setAllDates(date);
    } else if (this.value) {
      this.value.hepatitisBDate = date;
    }
  }

  private areAllDatesUndefined(): boolean {
    if (this.value) {
      return this.value.hepatitisBDate == null &&
        this.value.hivDate == null &&
        this.value.syphilisDate == null &&
        this.value.chlamydiaDate == null &&
        this.value.gonorrheaDate == null &&
        this.value.hemoglobinopathyDate == null;
    }
    return false;
  }

  private setAllDates(date: Date): void {
    if (this.value) {
      // We manually calculate offset date since we avoid triggering the data-card wrappers setDateOffsetOnPath functionality
      const offSetDate = calculateOffsetDate(date);
      this.value.hepatititsBSection = { dirty: true, date: offSetDate};
      this.value.hivSection = { dirty: true, date: offSetDate};
      this.value.syphilisSection = { dirty: true, date: offSetDate};
      this.value.chlamydiaSection = { dirty: true, date: offSetDate};
      this.value.gonorrheaSection = { dirty: true, date: offSetDate};
      this.value.hemoglobinopathySection = { dirty: true, date: offSetDate};
    }
  }
}
