import ActivityUtil from './activity.util';
import { Activity, ObservationType } from '@nspop/gm-web-facade-api';
import { ClinicalObservationTypeEnum, ScanningObservationTypeEnum } from './observation.util';

function createActivity(observationTypes?: ObservationType[]): Activity {
  const observations = observationTypes?.map(type => ({ type }));
  return { observations } as Activity;
}

describe('ActivityUtil', () => {
  const regularMeasurements: ObservationType[] = [ ClinicalObservationTypeEnum.BLOOD_PRESSURE ];
  const ultraSoundMeasurements: ObservationType[] = [ ScanningObservationTypeEnum.CERVIX ];

  describe('hasMeasurements', () => {
    it('returns true when there are observations', () => {
      const activity = createActivity(regularMeasurements);
      expect(ActivityUtil.hasMeasurements(activity)).toBe(true);
    });

    it('returns false when there is no activity', () => {
      expect(ActivityUtil.hasMeasurements(undefined)).toBe(false);
    });

    it('returns false when there is no observations', () => {
      const activity = createActivity();
      expect(ActivityUtil.hasMeasurements(activity)).toBe(false);
    });

    it('returns false when there are no observations', () => {
      const activity = createActivity([]);
      expect(ActivityUtil.hasMeasurements(activity)).toBe(false);
    });
  });

  describe('hasRegularMeasurements', () => {
    it('returns true when there are regular observations', () => {
      const activity = createActivity(regularMeasurements);
      expect(ActivityUtil.hasRegularMeasurements(activity)).toBe(true);
    });

    it('returns false when there is no activity', () => {
      expect(ActivityUtil.hasRegularMeasurements(undefined)).toBe(false);
    });

    it('returns false when there is no observations', () => {
      const activity = createActivity();
      expect(ActivityUtil.hasRegularMeasurements(activity)).toBe(false);
    });

    it('returns false when there are no regular observations', () => {
      const activity = createActivity(ultraSoundMeasurements);
      expect(ActivityUtil.hasRegularMeasurements(activity)).toBe(false);
    });
  });

  describe('hasUltraSoundMeasurements', () => {
    it('returns true when there are ultrasound observations', () => {
      const activity = createActivity(ultraSoundMeasurements);
      expect(ActivityUtil.hasUltraSoundMeasurements(activity)).toBe(true);
    });

    it('returns false when there is no activity', () => {
      expect(ActivityUtil.hasUltraSoundMeasurements(undefined)).toBe(false);
    });

    it('returns false when there is no observations', () => {
      const activity = createActivity();
      expect(ActivityUtil.hasUltraSoundMeasurements(activity)).toBe(false);
    });

    it('returns false when there are no ultrasound observations', () => {
      const activity = createActivity(regularMeasurements);
      expect(ActivityUtil.hasUltraSoundMeasurements(activity)).toBe(false);
    });
  });
});
